
PITAKA_GROUPS = {
    'sutta pitaka': {

        'digha nikaya': {
            'silakkhandha vagga': {},
            'maha vagga': {},
            'patika vagga': {},
        },

        'majjhima nikaya': {
            'mula pannasa': [
                'mulapariyaya vagga',
                'sihanada vagga',
                'opamma vagga',
                'mahayamaka vagga',
                'culayamaka vagga',
            ],
            'majjhima pannasa': [
                'gahapati vagga',
                'bhikkhu vagga',
                'paribbajaka vagga',
                'raja vagga',
                'brahmana vagga',
            ],
            'upari pannasa': [
                'devadaha vagga',
                'anupada vagga',
                'sunnata vagga',
                'vibhanga vagga',
                'salayatana vagga',
            ],
        },

        'samyutta nikaya': {
            'sagatha vagga': [
                'sn 1', 'sn 2', 'sn 3', 'sn 4', 'sn 5', 'sn 6', 'sn 7', 'sn 8', 'sn 9', 'sn 10', 'sn 11',
            ],
            'nidana vagga': [
                'sn 12', 'sn 13', 'sn 14', 'sn 15', 'sn 16', 'sn 17', 'sn 18', 'sn 19', 'sn 20', 'sn 21',
            ],
            'khandha vagga': [
                'sn 22', 'sn 23', 'sn 24', 'sn 25', 'sn 26', 'sn 27', 'sn 28', 'sn 29', 'sn 30', 'sn 31', 'sn 32',
                'sn 33', 'sn 34',
            ],
            'salayatana vagga': [
                'sn 35', 'sn 36', 'sn 37', 'sn 38', 'sn 39', 'sn 40', 'sn 41', 'sn 42', 'sn 43', 'sn 44',
            ],
            'maha vagga': [
                'sn 45', 'sn 46', 'sn 47', 'sn 48', 'sn 49', 'sn 50', 'sn 51', 'sn 52', 'sn 53', 'sn 54', 'sn 55',
                'sn 56',
            ],
        },
        'anguttara nikaya': {},
        'khuddaka nikaya': {},
    },

    'vinaya pitaka': {},

    'abhidhamma pitaka': {},
}
