# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'simsapa/assets/ui/import_stardict_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_ImportStarDictDialog(object):
    def setupUi(self, ImportStarDictDialog):
        ImportStarDictDialog.setObjectName("ImportStarDictDialog")
        ImportStarDictDialog.resize(500, 430)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(ImportStarDictDialog.sizePolicy().hasHeightForWidth())
        ImportStarDictDialog.setSizePolicy(sizePolicy)
        ImportStarDictDialog.setBaseSize(QtCore.QSize(500, 400))
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(ImportStarDictDialog)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.open_btn = QtWidgets.QPushButton(ImportStarDictDialog)
        self.open_btn.setObjectName("open_btn")
        self.verticalLayout.addWidget(self.open_btn, 0, QtCore.Qt.AlignTop)
        self.info = QtWidgets.QLabel(ImportStarDictDialog)
        self.info.setObjectName("info")
        self.verticalLayout.addWidget(self.info)
        spacerItem = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem)
        self.tabWidget = QtWidgets.QTabWidget(ImportStarDictDialog)
        self.tabWidget.setObjectName("tabWidget")
        self.create_new_tab = QtWidgets.QWidget()
        self.create_new_tab.setObjectName("create_new_tab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.create_new_tab)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.label_3 = QtWidgets.QLabel(self.create_new_tab)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.title = QtWidgets.QLineEdit(self.create_new_tab)
        self.title.setMinimumSize(QtCore.QSize(0, 40))
        self.title.setObjectName("title")
        self.verticalLayout_2.addWidget(self.title)
        self.label_4 = QtWidgets.QLabel(self.create_new_tab)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_2.addWidget(self.label_4)
        self.label = QtWidgets.QLineEdit(self.create_new_tab)
        self.label.setMinimumSize(QtCore.QSize(0, 40))
        self.label.setObjectName("label")
        self.verticalLayout_2.addWidget(self.label)
        self.label_5 = QtWidgets.QLabel(self.create_new_tab)
        self.label_5.setObjectName("label_5")
        self.verticalLayout_2.addWidget(self.label_5)
        self.import_new_btn = QtWidgets.QPushButton(self.create_new_tab)
        self.import_new_btn.setEnabled(False)
        self.import_new_btn.setCheckable(False)
        self.import_new_btn.setObjectName("import_new_btn")
        self.verticalLayout_2.addWidget(self.import_new_btn)
        spacerItem1 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.verticalLayout_3.addLayout(self.verticalLayout_2)
        self.tabWidget.addTab(self.create_new_tab, "")
        self.update_existing_tab = QtWidgets.QWidget()
        self.update_existing_tab.setObjectName("update_existing_tab")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.update_existing_tab)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.label_6 = QtWidgets.QLabel(self.update_existing_tab)
        self.label_6.setObjectName("label_6")
        self.verticalLayout_4.addWidget(self.label_6)
        self.select_title = QtWidgets.QComboBox(self.update_existing_tab)
        self.select_title.setObjectName("select_title")
        self.verticalLayout_4.addWidget(self.select_title)
        self.selected_info = QtWidgets.QLabel(self.update_existing_tab)
        self.selected_info.setObjectName("selected_info")
        self.verticalLayout_4.addWidget(self.selected_info)
        spacerItem2 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem2)
        self.update_existing_btn = QtWidgets.QPushButton(self.update_existing_tab)
        self.update_existing_btn.setEnabled(False)
        self.update_existing_btn.setObjectName("update_existing_btn")
        self.verticalLayout_4.addWidget(self.update_existing_btn)
        self.verticalLayout_5.addLayout(self.verticalLayout_4)
        self.tabWidget.addTab(self.update_existing_tab, "")
        self.verticalLayout.addWidget(self.tabWidget, 0, QtCore.Qt.AlignBottom)
        self.msg = QtWidgets.QLabel(ImportStarDictDialog)
        self.msg.setObjectName("msg")
        self.verticalLayout.addWidget(self.msg)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.close_btn = QtWidgets.QPushButton(ImportStarDictDialog)
        self.close_btn.setObjectName("close_btn")
        self.horizontalLayout.addWidget(self.close_btn)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2.addLayout(self.verticalLayout)

        self.retranslateUi(ImportStarDictDialog)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(ImportStarDictDialog)

    def retranslateUi(self, ImportStarDictDialog):
        _translate = QtCore.QCoreApplication.translate
        ImportStarDictDialog.setWindowTitle(_translate("ImportStarDictDialog", "Import from a StarDict Zip Dictionary"))
        self.open_btn.setText(_translate("ImportStarDictDialog", "Select StarDict ZIP..."))
        self.info.setText(_translate("ImportStarDictDialog", "info"))
        self.label_3.setText(_translate("ImportStarDictDialog", "Title:"))
        self.label_4.setText(_translate("ImportStarDictDialog", "Unique short label:"))
        self.label_5.setText(_translate("ImportStarDictDialog", "<html><head/><body><p>The label (such as NCPED) is used to distinguish the same words</p><p>in different dictionaries.</p></body></html>"))
        self.import_new_btn.setText(_translate("ImportStarDictDialog", "Import as new"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.create_new_tab), _translate("ImportStarDictDialog", "Import as new"))
        self.label_6.setText(_translate("ImportStarDictDialog", "Select title:"))
        self.selected_info.setText(_translate("ImportStarDictDialog", "selected info"))
        self.update_existing_btn.setText(_translate("ImportStarDictDialog", "Update existing"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.update_existing_tab), _translate("ImportStarDictDialog", "Update existing"))
        self.msg.setText(_translate("ImportStarDictDialog", "status msg"))
        self.close_btn.setText(_translate("ImportStarDictDialog", "Close"))
