"""Baseline

Revision ID: 5aaa36ccfa37
Revises: 
Create Date: 2021-12-21 14:39:34.115783

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = '5aaa36ccfa37'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    pass


def downgrade():
    pass
